@@ -578,7 +578,7 @@ fn display_tree_inner(
     lines: &mut Vec<String>,
     depth: usize,
 ) {
-    let prefix = "  ".repeat(depth).to_string();
+    let prefix = "  ".repeat(depth);
     match error {
         DerivationTree::Derived(derived) => {
             display_tree_inner(&derived.cause1, lines, depth + 1);
