@@ -159,7 +159,7 @@ impl<Context: BuildContext> ResolverProvider for DefaultResolverProvider<'_, Con
             .fetcher
             .client()
             .manual(|client, semaphore| {
-                client.package_metadata(
+                client.simple_detail(
                     package_name,
                     index.map(IndexMetadataRef::from),
                     self.capabilities,
@@ -200,7 +200,7 @@ impl<Context: BuildContext> ResolverProvider for DefaultResolverProvider<'_, Con
                     .collect(),
             )),
             Err(err) => match err.kind() {
-                uv_client::ErrorKind::PackageNotFound(_) => {
+                uv_client::ErrorKind::RemotePackageNotFound(_) => {
                     if let Some(flat_index) = flat_index
                         .and_then(|flat_index| flat_index.get(package_name))
                         .cloned()
