@@ -11,7 +11,6 @@ use itertools::Itertools;
 use owo_colors::OwoColorize;
 use rustc_hash::{FxBuildHasher, FxHashMap};
 use tracing::{debug, warn};
-use url::Url;
 
 use uv_cache::Cache;
 use uv_cache_key::RepositoryUrl;
@@ -68,8 +67,13 @@ pub(crate) async fn add(
     active: Option<bool>,
     no_sync: bool,
     no_install_project: bool,
+    only_install_project: bool,
     no_install_workspace: bool,
+    only_install_workspace: bool,
     no_install_local: bool,
+    only_install_local: bool,
+    no_install_package: Vec<PackageName>,
+    only_install_package: Vec<PackageName>,
     requirements: Vec<RequirementsSource>,
     constraints: Vec<RequirementsSource>,
     marker: Option<MarkerTree>,
@@ -744,8 +748,13 @@ pub(crate) async fn add(
         sync_state,
         lock_check,
         no_install_project,
+        only_install_project,
         no_install_workspace,
+        only_install_workspace,
         no_install_local,
+        only_install_local,
+        no_install_package.clone(),
+        only_install_package.clone(),
         &defaulted_extras,
         &defaulted_groups,
         raw,
@@ -975,8 +984,13 @@ async fn lock_and_sync(
     sync_state: PlatformState,
     lock_check: LockCheck,
     no_install_project: bool,
+    only_install_project: bool,
     no_install_workspace: bool,
+    only_install_workspace: bool,
     no_install_local: bool,
+    only_install_local: bool,
+    no_install_package: Vec<PackageName>,
+    only_install_package: Vec<PackageName>,
     extras: &ExtrasSpecificationWithDefaults,
     groups: &DependencyGroupsWithDefaults,
     raw: bool,
@@ -990,24 +1004,26 @@ async fn lock_and_sync(
     printer: Printer,
     preview: Preview,
 ) -> Result<(), ProjectError> {
-    let mut lock = project::lock::LockOperation::new(
-        if let LockCheck::Enabled(lock_check) = lock_check {
-            LockMode::Locked(target.interpreter(), lock_check)
-        } else {
-            LockMode::Write(target.interpreter())
-        },
-        &settings.resolver,
-        client_builder,
-        &lock_state,
-        Box::new(DefaultResolveLogger),
-        concurrency,
-        cache,
-        &WorkspaceCache::default(),
-        printer,
-        preview,
+    let mut lock = Box::pin(
+        project::lock::LockOperation::new(
+            if let LockCheck::Enabled(lock_check) = lock_check {
+                LockMode::Locked(target.interpreter(), lock_check)
+            } else {
+                LockMode::Write(target.interpreter())
+            },
+            &settings.resolver,
+            client_builder,
+            &lock_state,
+            Box::new(DefaultResolveLogger),
+            concurrency,
+            cache,
+            &WorkspaceCache::default(),
+            printer,
+            preview,
+        )
+        .with_constraints(constraints)
+        .execute((&target).into()),
     )
-    .with_constraints(constraints)
-    .execute((&target).into())
     .await?
     .into_lock();
 
@@ -1102,8 +1118,7 @@ async fn lock_and_sync(
 
             // Invalidate the project metadata.
             if let AddTarget::Project(VirtualProject::Project(ref project), _) = target {
-                let url = Url::from_file_path(project.project_root())
-                    .map(DisplaySafeUrl::from)
+                let url = DisplaySafeUrl::from_file_path(project.project_root())
                     .expect("project root is a valid URL");
                 let version_id = VersionId::from_url(&url);
                 let existing = lock_state.index().distributions().remove(&version_id);
@@ -1112,23 +1127,25 @@ async fn lock_and_sync(
 
             // If the file was modified, we have to lock again, though the only expected change is
             // the addition of the minimum version specifiers.
-            lock = project::lock::LockOperation::new(
-                if let LockCheck::Enabled(lock_check) = lock_check {
-                    LockMode::Locked(target.interpreter(), lock_check)
-                } else {
-                    LockMode::Write(target.interpreter())
-                },
-                &settings.resolver,
-                client_builder,
-                &lock_state,
-                Box::new(SummaryResolveLogger),
-                concurrency,
-                cache,
-                &WorkspaceCache::default(),
-                printer,
-                preview,
+            lock = Box::pin(
+                project::lock::LockOperation::new(
+                    if let LockCheck::Enabled(lock_check) = lock_check {
+                        LockMode::Locked(target.interpreter(), lock_check)
+                    } else {
+                        LockMode::Write(target.interpreter())
+                    },
+                    &settings.resolver,
+                    client_builder,
+                    &lock_state,
+                    Box::new(SummaryResolveLogger),
+                    concurrency,
+                    cache,
+                    &WorkspaceCache::default(),
+                    printer,
+                    preview,
+                )
+                .execute((&target).into()),
             )
-            .execute((&target).into())
             .await?
             .into_lock();
         }
@@ -1165,9 +1182,13 @@ async fn lock_and_sync(
         None,
         InstallOptions::new(
             no_install_project,
+            only_install_project,
             no_install_workspace,
+            only_install_workspace,
             no_install_local,
-            vec![],
+            only_install_local,
+            no_install_package,
+            only_install_package,
         ),
         Modifications::Sufficient,
         None,
